@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -563,6 +563,7 @@ static void rewrite_nofast_bytecodes_and_calculate_fingerprints() {
   }
 }
 
+#if INCLUDE_JVMTI
 static void relocate_cached_class_file() {
   for (int i = 0; i < _global_klass_objects->length(); i++) {
     Klass* k = _global_klass_objects->at(i);
@@ -579,6 +580,7 @@ static void relocate_cached_class_file() {
     }
   }
 }
+#endif // INCLUDE_JVMTI
 
 NOT_PRODUCT(
 static void assert_not_anonymous_class(InstanceKlass* k) {
@@ -842,6 +844,11 @@ public:
     do_ptr(&ptr);
   }
 
+  void do_bool(bool *p) {
+    void* ptr = (void*)(uintx(*p));
+    do_ptr(&ptr);
+  }
+
   void do_tag(int tag) {
     _dump_region->append_intptr_t((intptr_t)tag);
   }
@@ -1190,10 +1197,6 @@ private:
 
 public:
   static void copy_and_compact() {
-    // We should no longer allocate anything from the metaspace, so that
-    // we can have a stable set of MetaspaceObjs to work with.
-    Metaspace::freeze();
-
     ResourceMark rm;
     SortedSymbolClosure the_ssc; // StackObj
     _ssc = &the_ssc;
@@ -1347,6 +1350,14 @@ char* VM_PopulateDumpSharedSpace::dump_read_only_tables() {
 }
 
 void VM_PopulateDumpSharedSpace::doit() {
+  // We should no longer allocate anything from the metaspace, so that:
+  //
+  // (1) Metaspace::allocate might trigger GC if we have run out of
+  //     committed metaspace, but we can't GC because we're running
+  //     in the VM thread.
+  // (2) ArchiveCompactor needs to work with a stable set of MetaspaceObjs.
+  Metaspace::freeze();
+
   Thread* THREAD = VMThread::vm_thread();
 
   FileMapInfo::check_nonempty_dir_in_shared_path_table();
@@ -1411,8 +1422,6 @@ void VM_PopulateDumpSharedSpace::doit() {
   // the any dictionaries.
   NOT_PRODUCT(assert_no_anonymoys_classes_in_dictionaries());
 
-  SystemDictionaryShared::finalize_verification_constraints();
-
   ArchiveCompactor::initialize();
   ArchiveCompactor::copy_and_compact();
 
@@ -1433,7 +1442,7 @@ void VM_PopulateDumpSharedSpace::doit() {
   _md_region.pack(&_od_region);
 
   // Relocate the archived class file data into the od region
-  relocate_cached_class_file();
+  JVMTI_ONLY(relocate_cached_class_file();)
   _od_region.pack();
 
   // The 5 core spaces are allocated consecutively mc->rw->ro->md->od, so there total size
@@ -1658,7 +1667,6 @@ void MetaspaceShared::preload_and_dump(TRAPS) {
     ResourceMark rm;
     char class_list_path_str[JVM_MAXPATHLEN];
     // Preload classes to be shared.
-    // Should use some os:: method rather than fopen() here. aB.
     const char* class_list_path;
     if (SharedClassListFile == NULL) {
       // Construct the path to the class list (in jre/lib)
@@ -1713,6 +1721,8 @@ void MetaspaceShared::preload_and_dump(TRAPS) {
     SystemDictionary::clear_invoke_method_table();
     HeapShared::init_archivable_static_fields(THREAD);
 
+    SystemDictionaryShared::finalize_verification_constraints();
+
     VM_PopulateDumpSharedSpace op;
     VMThread::execute(&op);
   }
@@ -2001,6 +2011,11 @@ public:
     *p = (u4)(uintx(obj));
   }
 
+  void do_bool(bool* p) {
+    intptr_t obj = nextPtr();
+    *p = (bool)(uintx(obj));
+  }
+
   void do_tag(int tag) {
     int old_tag;
     old_tag = (int)(intptr_t)nextPtr();
