@@ -69,7 +69,7 @@ public class ClhsdbInspect {
                           "instance of Oop for java/lang/Class");
             tokensMap.put("Method*=", "Type is Method");
             tokensMap.put("(a java.lang.ref.ReferenceQueue$Lock)",
-                          "instance of Oop for java/lang/ref/ReferenceQueue$Lock");
+                          "instance of Oop for java/lang/ref/ReferenceQueue\\$Lock");
 
             String[] lines = jstackOutput.split("\\R");
 
