@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,7 +21,6 @@
  * questions.
  */
 
-
 // common infrastructure for SunPKCS11 tests
 
 import java.io.BufferedReader;
@@ -32,6 +31,10 @@ import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.StringReader;
 import java.nio.charset.StandardCharsets;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
+import java.nio.file.StandardCopyOption;
 import java.security.AlgorithmParameters;
 import java.security.InvalidAlgorithmParameterException;
 import java.security.KeyPairGenerator;
@@ -69,6 +72,8 @@ public abstract class PKCS11Test {
     // directory of the test source
     static final String BASE = System.getProperty("test.src", ".");
 
+    static final String TEST_CLASSES = System.getProperty("test.classes", ".");
+
     static final char SEP = File.separatorChar;
 
     private static final String DEFAULT_POLICY =
@@ -104,11 +109,9 @@ public abstract class PKCS11Test {
     // for quick checking for generic testing than many if-else statements.
     static double softoken3_version = -1;
     static double nss3_version = -1;
-    static Provider pkcs11;
+    static Provider pkcs11 = newPKCS11Provider();
 
-    // Goes through ServiceLoader instead of Provider.getInstance() since it
-    // works on all platforms
-    static {
+    public static Provider newPKCS11Provider() {
         ServiceLoader sl = ServiceLoader.load(java.security.Provider.class);
         Iterator<Provider> iter = sl.iterator();
         Provider p = null;
@@ -133,7 +136,7 @@ public abstract class PKCS11Test {
                 ex.printStackTrace();
             }
         }
-        pkcs11 = p;
+        return p;
     }
 
     /*
@@ -149,12 +152,17 @@ public abstract class PKCS11Test {
         return false;
     }
 
-    // Return a SunPKCS11 provider configured with the specified config file
+    // Return the static test SunPKCS11 provider configured with the specified config file
     static Provider getSunPKCS11(String config) throws Exception {
-        if (pkcs11 == null) {
+        return getSunPKCS11(config, pkcs11);
+    }
+
+    // Return the Provider p configured with the specified config file
+    static Provider getSunPKCS11(String config, Provider p) throws Exception {
+        if (p == null) {
             throw new NoSuchProviderException("No PKCS11 provider available");
         }
-        return pkcs11.configure(config);
+        return p.configure(config);
     }
 
     public abstract void main(Provider p) throws Exception;
@@ -200,7 +208,7 @@ public abstract class PKCS11Test {
                     test.enableSM = true;
                 } else {
                     throw new RuntimeException("Unknown Command, use 'sm' as "
-                            + "first arguemtn to enable security manager");
+                            + "first argument to enable security manager");
                 }
             }
             if (test.enableSM) {
@@ -532,36 +540,45 @@ public abstract class PKCS11Test {
         nss_library = "nss3";
     }
 
+    // Run NSS testing on a Provider p configured with test nss config
     public static void testNSS(PKCS11Test test) throws Exception {
+        String nssConfig = getNssConfig();
+        if (nssConfig == null) {
+            // issue loading libraries
+            return;
+        }
+        Provider p = getSunPKCS11(nssConfig);
+        test.premain(p);
+    }
+
+    public static String getNssConfig() throws Exception {
         String libdir = getNSSLibDir();
         if (libdir == null) {
-            return;
+            return null;
         }
-        String base = getBase();
 
         if (loadNSPR(libdir) == false) {
-            return;
+            return null;
         }
 
+        String base = getBase();
+
         String libfile = libdir + System.mapLibraryName(nss_library);
 
         String customDBdir = System.getProperty("CUSTOM_DB_DIR");
         String dbdir = (customDBdir != null) ?
-                                customDBdir :
-                                base + SEP + "nss" + SEP + "db";
+                customDBdir :
+                base + SEP + "nss" + SEP + "db";
         // NSS always wants forward slashes for the config path
         dbdir = dbdir.replace('\\', '/');
 
         String customConfig = System.getProperty("CUSTOM_P11_CONFIG");
         String customConfigName = System.getProperty("CUSTOM_P11_CONFIG_NAME", "p11-nss.txt");
-        String p11config = (customConfig != null) ?
-                                customConfig :
-                                base + SEP + "nss" + SEP + customConfigName;
-
         System.setProperty("pkcs11test.nss.lib", libfile);
         System.setProperty("pkcs11test.nss.db", dbdir);
-        Provider p = getSunPKCS11(p11config);
-        test.premain(p);
+        return (customConfig != null) ?
+                customConfig :
+                base + SEP + "nss" + SEP + customConfigName;
     }
 
     // Generate a vector of supported elliptic curves of a given provider
@@ -690,7 +707,8 @@ public abstract class PKCS11Test {
                 "/usr/lib/arm-linux-gnueabihf/nss/" });
         osMap.put("Linux-aarch64-64", new String[] {
                 "/usr/lib/aarch64-linux-gnu/",
-                "/usr/lib/aarch64-linux-gnu/nss/" });
+                "/usr/lib/aarch64-linux-gnu/nss/",
+                "/usr/lib64/" });
         return osMap;
     }
 
@@ -891,6 +909,31 @@ public abstract class PKCS11Test {
         return path;
     }
 
+    protected void setCommonSystemProps() {
+        System.setProperty("java.security.debug", "true");
+        System.setProperty("NO_DEIMOS", "true");
+        System.setProperty("NO_DEFAULT", "true");
+        System.setProperty("CUSTOM_DB_DIR", TEST_CLASSES);
+    }
+
+    protected void copyNssCertKeyToClassesDir() throws IOException {
+        Path dbPath = Path.of(BASE).getParent().resolve("nss").resolve("db");
+        copyNssCertKeyToClassesDir(dbPath);
+    }
+
+    protected void copyNssCertKeyToClassesDir(Path dbPath) throws IOException {
+        Path destinationPath = Path.of(TEST_CLASSES);
+        String keyDbFile = "key3.db";
+        String certDbFile = "cert8.db";
+
+        Files.copy(dbPath.resolve(certDbFile),
+                destinationPath.resolve(certDbFile),
+                StandardCopyOption.REPLACE_EXISTING);
+        Files.copy(dbPath.resolve(keyDbFile),
+                destinationPath.resolve(keyDbFile),
+                StandardCopyOption.REPLACE_EXISTING);
+    }
+
     @Artifact(
             organization = "jpg.tests.jdk.nsslib",
             name = "nsslib-windows_x64",
