@@ -154,12 +154,22 @@ public class Test7029048 extends TestHelper {
                     }
 
                     desc = "LD_LIBRARY_PATH should not be set (no libjvm.so)";
+                    if (TestHelper.isAIX) {
+                        System.out.println("Skipping test case \"" + desc +
+                                           "\" because the Aix launcher adds the paths in any case.");
+                        continue;
+                    }
                     break;
                 case NO_DIR:
                     if (dstLibDir.exists()) {
                         recursiveDelete(dstLibDir);
                     }
                     desc = "LD_LIBRARY_PATH should not be set (no directory)";
+                    if (TestHelper.isAIX) {
+                        System.out.println("Skipping test case \"" + desc +
+                                           "\" because the Aix launcher adds the paths in any case.");
+                        continue;
+                    }
                     break;
                 default:
                     throw new RuntimeException("unknown case");
