@@ -23,17 +23,18 @@
 
 /*
  * @test
- * @bug 8153716 8143955 8151754 8150382 8153920 8156910 8131024 8160089 8153897 8167128 8154513 8170015 8170368 8172102 8172103  8165405 8173073 8173848 8174041 8173916 8174028 8174262 8174797 8177079 8180508 8177466 8172154 8192979 8191842 8198573 8198801
+ * @bug 8153716 8143955 8151754 8150382 8153920 8156910 8131024 8160089 8153897 8167128 8154513 8170015 8170368 8172102 8172103  8165405 8173073 8173848 8174041 8173916 8174028 8174262 8174797 8177079 8180508 8177466 8172154 8192979 8191842 8198573 8198801 8239536 8210959
  * @summary Simple jshell tool tests
  * @modules jdk.compiler/com.sun.tools.javac.api
  *          jdk.compiler/com.sun.tools.javac.main
  *          jdk.jdeps/com.sun.tools.javap
  *          jdk.jshell/jdk.internal.jshell.tool
  * @build KullaTesting TestingInputStream
- * @run testng ToolSimpleTest
+ * @run testng/othervm ToolSimpleTest
  */
-import java.util.Arrays;
+
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 import java.util.Locale;
 import java.util.function.Consumer;
@@ -112,6 +113,20 @@ public class ToolSimpleTest extends ReplToolTesting {
         );
     }
 
+    @Test
+    public void testThrowWithPercent() {
+        test(
+                (a) -> assertCommandCheckOutput(a,
+                        "URI u = new URI(\"http\", null, \"h\", -1, \"a\" + (char)0x04, null, null);", (s) ->
+                                assertTrue(s.contains("URISyntaxException") && !s.contains("JShellTool"),
+                                        "Output: '" + s + "'")),
+                (a) -> assertCommandCheckOutput(a,
+                        "throw new Exception(\"%z\")", (s) ->
+                                assertTrue(s.contains("java.lang.Exception") && !s.contains("UnknownFormatConversionException"),
+                                        "Output: '" + s + "'"))
+        );
+    }
+
     @Test
     public void oneLineOfError() {
         test(
@@ -830,4 +845,20 @@ public class ToolSimpleTest extends ReplToolTesting {
                 a -> assertCommandOutputContains(a, "a", "A@")
         );
     }
+
+    @Test
+    public void testImportChange() {
+        for (String feedback : new String[] {"verbose", "normal"}) {
+            test(
+                    (a) -> assertCommandOutputContains(a, "/set feedback " + feedback, "|  Feedback mode: " + feedback),
+                    (a) -> assertCommand(a, "import java.util.*", ""),
+                    (a) -> assertCommandOutputContains(a, "var v1 = List.of(1);", "v1 ==> [1]"),
+                    (a) -> assertCommandOutputContains(a, "import java.awt.List;",
+                            "|    update replaced variable v1 which cannot be referenced until this error is corrected:"),
+                    (a) -> assertCommandOutputContains(a, "var b = java.util.List.of(\"bb\")",
+                            "b ==> [bb]"),
+                    (a) -> assertCommandOutputContains(a, "b", "b ==> [bb]")
+            );
+        }
+    }
 }
